var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Injectable } from "@angular/core";
import { each, get, isFunction, noop, set } from "lodash-es";
import { compileAngularJsComponent, getAngularJsInjectable } from "../../commons/angular-angularjs-utils";
import { TimeSliderComponent } from "../../commons/time-slider/time-slider.component";
import { SandSearchInputComponent } from "../../sand/search-input/sand-search-input.component";
import { ControlCollectionComponent } from "../../spb/page-viewer/control-collection/control-collection.component";
import { zoneWrap } from "../../commons/api-utils";
import { SearchAndSelectWithPreviewComponent } from "../../sand/search-and-select-with-preview/search-and-select-with-preview.component";
/* The details of our supported components. This maps each SviComponent to its associated selector
   or Angular Component class. This indirection allows us to change implementation details at a later
   date. */
export const components = {
    "svi-control-collection": {
        selector: "spb-control-collection",
        class: ControlCollectionComponent
    },
    "svi-relationship-details": {
        selector: "relate-to-details",
        fnBindings: {
            onLinkReasonSelected: { params: ["linkReason"] },
            setApi: { params: ["api"] },
            onRelatedPageLoad: { params: [] },
            onRelatedPageChange: { params: ["change"] },
            onRelationshipValuesChange: { params: ["current", "previous"] }
        },
        defaultBindings: {
            onRelatedPageLoad: () => {
                window.sas.vi?.component.detectChanges();
            }
        }
    },
    "svi-search-input": {
        selector: "sand-search-input",
        class: SandSearchInputComponent
    },
    "svi-page": {
        selector: "spb-sds-page-viewer",
        fnBindings: {
            setApi: { params: ["api"] },
            onDirty: { params: ["isDirty"] },
            onPageLoad: { params: [] },
            onChange: { params: ["change"] }
        },
        defaultBindings: {
            onPageLoad: () => {
                window.sas.vi?.component.detectChanges();
            }
        }
    },
    "svi-search-and-select-with-preview": {
        selector: "sand-search-and-select-with-preview",
        class: SearchAndSelectWithPreviewComponent
    },
    "svi-time-slider": {
        selector: "svi-time-slider",
        class: TimeSliderComponent
    }
};
let ComponentApiService = class ComponentApiService {
    constructor(componentFactoryResolver, injector, appRef, ngZone) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.injector = injector;
        this.appRef = appRef;
        this.ngZone = ngZone;
        this.$timeout = () => getAngularJsInjectable("$timeout");
        this.getApi = () => {
            return {
                create: zoneWrap(this.create.bind(this), this.ngZone),
                detectChanges: this.detectChanges
            };
        };
        this.detectChanges = (callback = noop) => {
            // $timeout for AngularJS
            this.$timeout()(() => {
                // ngZone.run() for Angular. A single $timeout is not enough in some circumstances.
                this.ngZone.run(callback);
            });
        };
        this.compileComponent = (component, target, bindings = {}) => {
            const details = components[component];
            bindings = { ...bindings, ...details.defaultBindings };
            if (details.class) {
                return this.compileAngularComponent(details.class, target, bindings);
            }
            return this.compileAngularJSComponent(component, target, bindings);
        };
        this.compileAngularJSComponent = (component, target, bindings = {}) => {
            const selector = components[component].selector;
            const attrs = this.getAngularJsAttrs(bindings, component);
            const { element, destroy, scope } = compileAngularJsComponent(target, selector, bindings, attrs, true);
            return Promise.resolve({ element, destroy, component: scope });
        };
        this.getAngularJsAttrs = (bindings, component) => {
            const attrs = {};
            each(bindings, (_v, k) => {
                const fnBinding = get(components[component].fnBindings, k);
                if (fnBinding) {
                    // The attribute value for function bindings (&) must be of the form:
                    //     on-foo="onFoo(arg1,arg2)"
                    // If not, the function will not be invoked. The argument names must
                    // also match those that the caller uses.
                    const params = fnBinding.params.join(",");
                    set(attrs, k, `${k}(${params})`);
                }
                else {
                    set(attrs, k, k);
                }
            });
            return attrs;
        };
    }
    create(component, target, bindings) {
        if (!(component in components)) {
            console.warn(`"${component}" is not a component that can be created.`);
            return Promise.reject();
        }
        return this.compileComponent(component, target, bindings).then((result) => {
            this.detectChanges();
            return result;
        });
    }
    compileAngularComponent(component, target, bindings) {
        const factory = this.componentFactoryResolver.resolveComponentFactory(component);
        const ref = factory.create(this.injector);
        const instance = ref.instance;
        const element = ref.hostView.rootNodes[0];
        const subscriptions = [];
        each(bindings, (v, k) => {
            if (isFunction(v)) {
                const sub = instance[k].subscribe(v);
                subscriptions.push(sub);
            }
            else {
                set(instance, k, v);
            }
        });
        this.appRef.attachView(ref.hostView);
        target.append(element);
        this.appRef.tick();
        return Promise.resolve({
            element,
            destroy: () => {
                this.appRef.detachView(ref.hostView);
                ref.destroy();
                each(subscriptions, s => s.unsubscribe());
                this.detectChanges();
            },
            component: instance
        });
    }
};
ComponentApiService = __decorate([
    Injectable()
], ComponentApiService);
export { ComponentApiService };
