var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Injectable } from "@angular/core";
import { map } from "rxjs/operators";
import { getAngularJsInjectable } from "../../commons/angular-angularjs-utils";
let ObjectApiService = class ObjectApiService {
    constructor(attachmentsService, childObjectViewerValidationService) {
        this.attachmentsService = attachmentsService;
        this.childObjectViewerValidationService = childObjectViewerValidationService;
        this.documentService = () => getAngularJsInjectable("spbDocumentService");
        this.documentDisplayFormatter = () => getAngularJsInjectable("spbDocumentDisplayFormatter");
        this.dataDictionaryService = () => getAngularJsInjectable("dataDictionaryService");
        this.getObjectAccessRules = (objectType, objectId) => {
            return {
                canReadObject: this.dataDictionaryService().canReadDocument(objectType, objectId),
                canUpdateObject: this.dataDictionaryService().canUpdateDocument(objectType, objectId),
                canDeleteObject: this.dataDictionaryService().canDeleteDocument(objectType, objectId)
            };
        };
        this.getObject = (objectType, objectId, includeDisplayLabel = true) => {
            return this.documentService().getDocument(objectType, objectId, { includeDisplayLabel });
        };
        this.getChildObjects = (objectType, objectId, childObjectType, maxResponses = 25) => {
            return this.documentService().getSubDocuments(objectId, objectType, childObjectType, { maxResponses });
        };
        this.updateObject = (objectType, objectId, objectTypeId, objectTypeVersion, fieldValues, options) => {
            return this.documentService().saveDocument(objectTypeId, objectType, objectTypeVersion, objectId, fieldValues, options?.fileOperations, options?.sheets, options?.isExternalObject);
        };
        this.deleteObject = (objectType, objectId) => {
            return this.documentService().deleteDocument(objectType, objectId);
        };
        this.createObject = (objectType, objectTypeId, fieldValues, options) => {
            return this.documentService().createDocument(objectTypeId, objectType, fieldValues, options?.fileOperations, options?.comments, options?.sheets);
        };
        this.getObjectRelationship = (relationshipId) => {
            return this.documentService().getLink(relationshipId);
        };
        this.updateObjectRelationship = (relationshipId, payload) => {
            return this.documentService().saveLink(relationshipId, payload);
        };
        this.deleteObjectRelationship = (relationshipId) => {
            return this.documentService().deleteLink(relationshipId);
        };
        this.relateObject = (from, to, relationship) => {
            return new Promise((resolve, reject) => {
                this.documentService()
                    .linkDocument(this.convertObjectForType(from), this.convertObjectForType(to), this.convertObjectRelationshipForName(relationship))
                    .then(result => {
                    resolve(result.data);
                })
                    .catch(error => {
                    reject(error.data.message);
                });
            });
        };
        this.getObjectSummaryLabel = (objectType, objectId) => {
            return this.documentService().getDocumentSummaryLabel(objectType, objectId);
        };
        this.createAndRelateObject = (from, to, relationship) => {
            return new Promise((resolve, reject) => {
                this.documentService()
                    .createAndLinkDocument(this.convertObjectForType(from), this.convertObjectForData(to), this.convertObjectRelationshipForName(relationship))
                    .then(result => {
                    resolve(result.data);
                })
                    .catch(error => {
                    reject(error.data.message);
                });
            });
        };
        this.formatObjectFieldsForDisplay = (objects, objectType, options) => {
            return this.documentDisplayFormatter().processDocumentArray(objects, objectType, options?.skipFormatReferenceData ?? false, options?.skipFormatUserGroups ?? false, options?.skipFormatBooleans ?? false, options?.formatCallback);
        };
        this.uploadAttachment = (attachmentModel, objectType, objectId) => {
            return this.attachmentsService
                .createAttachment(attachmentModel, objectType, objectId)
                .pipe(map(result => result.body ?? {}))
                .toPromise()
                .catch(error => Promise.reject(error.message));
        };
        this.deleteAttachment = (objectType, objectId, attachmentId) => {
            return this.attachmentsService
                .removeAttachment(objectType, objectId, attachmentId)
                .toPromise()
                .then(() => Promise.resolve())
                .catch(error => Promise.reject(error.message));
        };
        this.getAttachment = (objectType, objectId, attachmentId) => {
            return this.attachmentsService
                .getAttachment(objectId, objectType, attachmentId)
                .pipe(map(result => result.body ?? {}))
                .toPromise()
                .catch(error => Promise.reject(error.message));
        };
        this.validate = (pageModel, childObject) => {
            return this.childObjectViewerValidationService
                .validate(pageModel, childObject)
                .toPromise();
        };
        /**
         * Edits the property names of the type to conform to AngularJS services currently injected.
         *
         * fieldValues to data
         * objectTypeName to name
         *
         * @param object {VIObject} The SAS Visual Investigator Object to be edited.
         * @return {ObjectForDocumentServiceRelationships}
         * @ignore
         *
         */
        this.convertObjectForData = (object) => {
            return {
                data: object.fieldValues,
                fileOperations: object.fileOperations,
                comments: object.comments,
                createdAt: object.createdAt,
                lastUpdatedAt: object.lastUpdatedAt,
                objectTypeId: object.objectTypeId,
                type: object.objectTypeName,
                sheets: object.sheets,
                id: object.id
            };
        };
        /**
         * Edits the property names of the type to conform to AngularJS services currently injected.
         *
         * objectTypeName to name
         *
         * @param object {VIObject} The SAS Visual Investigator Object to be edited.
         * @returns {ObjectForDocumentServiceRelationships}
         * @ignore
         */
        this.convertObjectForType = (object) => {
            return {
                fieldValues: object.fieldValues,
                fileOperations: object.fileOperations,
                comments: object.comments,
                createdAt: object.createdAt,
                lastUpdatedAt: object.lastUpdatedAt,
                objectTypeId: object.objectTypeId,
                type: object.objectTypeName,
                sheets: object.sheets,
                id: object.id
            };
        };
        /**
         * Edits the property names of the type to conform to AngularJS services currently injected.
         *
         * relationshipTypeName to name
         *
         * @param relationship {ObjectRelationship} Object Relationship to be edited.
         * @returns {RelationshipForDocumentServiceRelationships}
         * @ignore
         */
        this.convertObjectRelationshipForName = (relationship) => {
            return {
                name: relationship.relationshipTypeName,
                relationshipTypeLabel: relationship.relationshipTypeLabel,
                id: relationship.id,
                createdAt: relationship.createdAt,
                lastUpdatedAt: relationship.lastUpdatedAt,
                validFrom: relationship.validFrom,
                validTo: relationship.validTo,
                displayLabel: relationship.displayLabel,
                fromObjectTypeName: relationship.fromObjectTypeName,
                fromObjectTypeVersion: relationship.fromObjectTypeVersion,
                fromObjectId: relationship.fromObjectId,
                fromObjectDisplayLabel: relationship.fromObjectDisplayLabel,
                toObjectTypeName: relationship.toObjectTypeName,
                toObjectTypeVersion: relationship.toObjectTypeVersion,
                toObjectId: relationship.toObjectId,
                toObjectDisplayLabel: relationship.toObjectDisplayLabel,
                fieldValues: relationship.fieldValues,
                qualifiedTypeName: relationship.qualifiedTypeName
            };
        };
    }
    getApi() {
        return {
            getObjectRelationship: this.getObjectRelationship,
            updateObjectRelationship: this.updateObjectRelationship,
            deleteObjectRelationship: this.deleteObjectRelationship,
            getObject: this.getObject,
            updateObject: this.updateObject,
            deleteObject: this.deleteObject,
            createObject: this.createObject,
            getChildObjects: this.getChildObjects,
            relateObject: this.relateObject,
            getObjectSummaryLabel: this.getObjectSummaryLabel,
            createAndRelateObject: this.createAndRelateObject,
            formatObjectFieldsForDisplay: this.formatObjectFieldsForDisplay,
            uploadAttachment: this.uploadAttachment,
            deleteAttachment: this.deleteAttachment,
            getAttachment: this.getAttachment,
            getObjectAccessRules: this.getObjectAccessRules,
            validate: this.validate
        };
    }
};
ObjectApiService = __decorate([
    Injectable()
], ObjectApiService);
export { ObjectApiService };
