var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Injectable } from "@angular/core";
import { each } from "lodash-es";
import { getAngularJsInjectable } from "../../../commons/angular-angularjs-utils";
import { SEARCH_SELECTION_ID, VisualizationTypes } from "../../../sand/search/search";
import { zoneWrap } from "../../../commons/api-utils";
let ClientSearchApiService = class ClientSearchApiService {
    constructor(ngZone, searchApiService, sandRestService, sandDoSearchService, sandSearchStorageService, sandMetadataService, sandDocumentInfoService, searchAndCreateWizardService) {
        this.ngZone = ngZone;
        this.searchApiService = searchApiService;
        this.sandRestService = sandRestService;
        this.sandDoSearchService = sandDoSearchService;
        this.sandSearchStorageService = sandSearchStorageService;
        this.sandMetadataService = sandMetadataService;
        this.sandDocumentInfoService = sandDocumentInfoService;
        this.searchAndCreateWizardService = searchAndCreateWizardService;
        this.performSearch = (query) => {
            return new Promise(resolve => {
                this.sandRestService.search(query).subscribe(result => {
                    resolve(result);
                });
            });
        };
        this.navigateSearch = (queryText, visualizationName, mapFilters, queryMode) => {
            this.sandDoSearchService.doSearch(queryText, visualizationName, mapFilters, queryMode);
        };
        this.getCurrentSearchQuery = () => {
            return this.sandSearchStorageService.getCurrentSearchQuery();
        };
        this.createQueryStringFromModel = (queryModel) => {
            const sandQueryBuilderQueryService = getAngularJsInjectable("sandQueryBuilderQueryService");
            return this.sandMetadataService
                .getMetadata()
                .toPromise()
                .then(metadata => {
                return sandQueryBuilderQueryService.createQueryStringFromModel(queryModel, metadata);
            });
        };
        this.getQueryBuilderModel = (id) => {
            if (!id) {
                id = SEARCH_SELECTION_ID;
            }
            return this.sandSearchStorageService.getQueryBuilderModel(id);
        };
        this.setQueryBuilderModel = (queryModel, id) => {
            if (!id) {
                id = SEARCH_SELECTION_ID;
            }
            this.sandSearchStorageService.saveQueryBuilderModel(id, queryModel);
        };
        this.openAddObjectsToWorkspaceDialog = (visualizationName, objects, currentObject, networkData) => {
            this.sandDocumentSelectionDialogService = getAngularJsInjectable("sandDocumentSelectionDialogService");
            this.sandDocumentSelectionDialogService.openAddAllObjectsToWorkspaceDialog(visualizationName, currentObject, objects, networkData);
        };
        this.openAddImageToInsightsDialog = (imageData, contentType, height, width) => {
            this.sandDocumentSelectionDialogService = getAngularJsInjectable("sandDocumentSelectionDialogService");
            this.sandDocumentSelectionDialogService.openAddImageToInsightDialog(VisualizationTypes.Image, imageData, height ?? undefined, width ?? undefined, contentType);
        };
        this.addToNewWorkspace = (objectsToAdd, targetObjectType, targetObjectId, visualization, workspaceName, networkData) => {
            const worksheetService = getAngularJsInjectable("worksheetService");
            const sheetService = getAngularJsInjectable("sheetService");
            each(objectsToAdd, object => {
                this.sandDocumentInfoService.createUniqueKey(object);
            });
            const targetObject = {
                id: targetObjectId,
                type: targetObjectType,
                objectType: targetObjectType,
                isNewDocument: targetObjectId ? false : true
            };
            if (!targetObject.isNewDocument && !sheetService.getOpenDocument(targetObject.id, targetObject.type)) {
                throw Error(`Cannot add objects to workspace. Object of type ${targetObject.type} and id ${targetObject.id} is not open.`);
            }
            else {
                worksheetService.addToSheet(targetObject, undefined, objectsToAdd, visualization, networkData, workspaceName);
            }
        };
        this.openSearchAndCreateDialog = (wizardValues) => {
            const values = {
                ...wizardValues,
                searchFirst: true,
                initialCreateData: null
            };
            return this.searchAndCreateWizardService.launch(values).closeResult.toPromise();
        };
    }
    getApi() {
        return {
            ...this.searchApiService.getApi(),
            performSearch: this.performSearch,
            navigateSearch: zoneWrap(this.navigateSearch, this.ngZone),
            getCurrentSearchQuery: this.getCurrentSearchQuery,
            createQueryStringFromModel: this.createQueryStringFromModel,
            getQueryBuilderModel: this.getQueryBuilderModel,
            setQueryBuilderModel: this.setQueryBuilderModel,
            openAddObjectsToWorkspaceDialog: zoneWrap(this.openAddObjectsToWorkspaceDialog, this.ngZone),
            openAddImageToInsightsDialog: zoneWrap(this.openAddImageToInsightsDialog, this.ngZone),
            addToNewWorkspace: zoneWrap(this.addToNewWorkspace, this.ngZone),
            openSearchAndCreateDialog: zoneWrap(this.openSearchAndCreateDialog, this.ngZone)
        };
    }
};
ClientSearchApiService = __decorate([
    Injectable()
], ClientSearchApiService);
export { ClientSearchApiService };
