var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Injectable } from "@angular/core";
import { each } from "lodash-es";
import { getAngularJsInjectable } from "../../commons/angular-angularjs-utils";
import { VisualizationTypes } from "../../sand/search/search";
import { zoneWrap } from "../../commons/api-utils";
let SheetApiService = class SheetApiService {
    constructor(networkMenuHandlerService, ngZone, notificationService, resourceService, sandDocumentInfoService) {
        this.networkMenuHandlerService = networkMenuHandlerService;
        this.ngZone = ngZone;
        this.notificationService = notificationService;
        this.resourceService = resourceService;
        this.sandDocumentInfoService = sandDocumentInfoService;
        this.getOpenObject = (objectId, objectType) => {
            this.sheetService = getAngularJsInjectable("sheetService");
            return this.sheetService.getOpenDocument(objectId, objectType) ?? undefined;
        };
        this.getOpenObjects = () => {
            this.sheetService = getAngularJsInjectable("sheetService");
            return this.sheetService.getOpenDocuments();
        };
        this.getOpenObjectBySheet = (sheet) => {
            const worksheetService = getAngularJsInjectable("worksheetService");
            return worksheetService.getCurrentDocument(sheet);
        };
        this.createContextMenuForNodes = (workspace, handleAddToWorkspaceFn, handleAddToInsightsFn) => {
            return workspace.networkDiagramComponent.framework.view.constructNodeContextMenu(handleAddToWorkspaceFn, handleAddToInsightsFn).items;
        };
        this.registerNetworkMenuCallback = (menuType, callback) => {
            this.networkMenuHandlerService.registerNetworkMenuCallback(menuType, callback);
        };
        this.openAddAllObjectsToWorkspaceDialog = (objects, options) => {
            this.sandDocumentSelectionDialogService = getAngularJsInjectable("sandDocumentSelectionDialogService");
            this.sandDocumentSelectionDialogService.openAddAllObjectsToWorkspaceDialog(options?.visualizationName, options?.currentObject, objects, options?.networkData);
        };
        this.openAddImageToInsightsDialog = (imageData, contentType = "image/png", height, width) => {
            this.sandDocumentSelectionDialogService = getAngularJsInjectable("sandDocumentSelectionDialogService");
            if (imageData instanceof Uint8Array) {
                this.sandDocumentSelectionDialogService.openAddImageToInsightDialog(VisualizationTypes.Image, imageData, height, width, contentType);
            }
            else {
                const util = getAngularJsInjectable("Util");
                return this.imgToB64(imageData).then((image) => {
                    const imageBlob = util.b64toBlob(image);
                    this.sandDocumentSelectionDialogService.openAddImageToInsightDialog(VisualizationTypes.Image, imageBlob, undefined, undefined, contentType);
                });
            }
        };
        /**
         * Creates an HTML canvas for the img element.
         * Generates a new unscaled image based on the original image source.
         * If there is an error loading the new image, an error alert is displayed.
         *
         * @param element {HTMLElement} Element containing the image.
         * @returns {Promise<string>} A Promise which resolves to the Base64 string representing the image.
         * @ignore
         */
        this.imgToB64 = (element) => {
            const canvasElement = document.createElement("canvas");
            const canvasContext = canvasElement.getContext("2d");
            return new Promise(resolve => {
                if (canvasContext) {
                    const originalImageElement = element.querySelector("img");
                    const unscaledImage = new Image();
                    unscaledImage.crossOrigin = "Anonymous";
                    unscaledImage.onload = () => {
                        canvasElement.width = unscaledImage.width;
                        canvasElement.height = unscaledImage.height;
                        // An intermittent issue in IE11 means we use a timeout to move the drawimage out
                        // of the current execution
                        // https://stackoverflow.com/questions/25214395/unexpected-call-to-method-or-property-access-while-
                        // drawing-svg-image-onto-canvas
                        setTimeout(() => {
                            canvasContext.drawImage(unscaledImage, 0, 0, unscaledImage.width, unscaledImage.height);
                            resolve(canvasElement.toDataURL());
                            canvasElement.remove();
                        });
                    };
                    unscaledImage.onerror = () => {
                        this.notificationService.alertError(this.resourceService.get("spb.images.addToWorkspaceError.title"), this.resourceService.get("spb.images.addToWorkspaceError.txt"));
                        canvasElement.remove();
                    };
                    unscaledImage.src = originalImageElement.src;
                }
            });
        };
        this.addToNewWorkspace = (objectsToAdd, targetObjectType, options) => {
            const worksheetService = getAngularJsInjectable("worksheetService");
            const sheetService = getAngularJsInjectable("sheetService");
            each(objectsToAdd, object => {
                this.sandDocumentInfoService.createUniqueKey(object);
            });
            const targetObject = {
                id: options?.targetObjectId,
                type: targetObjectType,
                objectType: targetObjectType,
                isNewDocument: !options?.targetObjectId
            };
            if (!targetObject.isNewDocument &&
                targetObject.id &&
                !sheetService.getOpenDocument(targetObject.id, targetObject.type)) {
                console.warn(`Cannot add objects to workspace. Object of type ${targetObject.type} and id ${targetObject.id} is not open.`);
            }
            else {
                worksheetService.addToSheet(targetObject, undefined, objectsToAdd, options?.visualization, options?.networkData, options?.workspaceName);
            }
        };
        this.addToExistingWorkspace = (targetObject, workspaceClientId, objectsToAdd, networkData, isUndoRedo) => {
            const worksheetService = getAngularJsInjectable("worksheetService");
            each(objectsToAdd, object => {
                this.sandDocumentInfoService.createUniqueKey(object);
            });
            worksheetService.addToExistingWorksheet(targetObject, workspaceClientId, objectsToAdd, true, networkData, isUndoRedo);
        };
    }
    getApi() {
        return {
            getOpenObject: this.getOpenObject,
            getOpenObjects: this.getOpenObjects,
            getOpenObjectBySheet: this.getOpenObjectBySheet,
            createContextMenuForNodes: zoneWrap(this.createContextMenuForNodes, this.ngZone),
            registerNetworkMenuCallback: zoneWrap(this.registerNetworkMenuCallback, this.ngZone),
            openAddAllObjectsToWorkspaceDialog: zoneWrap(this.openAddAllObjectsToWorkspaceDialog, this.ngZone),
            openAddImageToInsightsDialog: zoneWrap(this.openAddImageToInsightsDialog, this.ngZone),
            addToExistingWorkspace: zoneWrap(this.addToExistingWorkspace, this.ngZone),
            addToNewWorkspace: zoneWrap(this.addToNewWorkspace, this.ngZone)
        };
    }
};
SheetApiService = __decorate([
    Injectable({
        providedIn: "root"
    })
], SheetApiService);
export { SheetApiService };
