var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Injectable } from "@angular/core";
import { map } from "rxjs/operators";
let TraversalApiService = class TraversalApiService {
    constructor(adminTraversalsClientService, pathService) {
        this.adminTraversalsClientService = adminTraversalsClientService;
        this.pathService = pathService;
        /**
         * Gets an array of Records from a list of given traversals filtered by destinationType.
         * The returned records will contain the values for fields specified in the fields param.
         * @param rootObjectId {string} Traversal's root object ID.
         * @param rootObjectType {string} Traversal's root object Type.
         * @param traversalUuids {string[]} Array of traversal UUIDs associated with the root object type.
         * @param destinationObjectType {string} Destination object's type.
         * @param fields {string[]} Names of the fields to be retrieved.
         * @param options {object} Optional object containing options: includeDisplayLabels.
         * @returns {Promise<SviRecord[]>} An array of SviRecords for the given Traversal.
         */
        this.getFieldsByTraversals = (rootObjectId, rootObjectType, traversalUuids, destinationObjectType, fields, options) => {
            return this.pathService
                .getRecordValuesByTraversals(rootObjectId, rootObjectType, traversalUuids, destinationObjectType, fields, options)
                .toPromise()
                .then((result) => {
                return result || [];
            })
                .catch((err) => Promise.reject(err?.message));
        };
        this.getTraversals = () => {
            return this.adminTraversalsClientService
                .getTraversals()
                .pipe(map(result => (result.body ? this.addTraversalEndPoint(result.body) : [])))
                .toPromise()
                .catch(err => Promise.reject(err?.error?.message));
        };
        /**
         * Adds the traversal end point to the top level of the object.
         * @param traversals {Traversal[]} Traversals that need end points added.
         * @returns {TraversalWithEndpoint[]} Array of traversals with end points.
         * @ignore
         */
        this.addTraversalEndPoint = (traversals) => {
            traversals?.forEach((traversal) => {
                let traversalBranch = traversal.traversalDefinition;
                if (traversalBranch) {
                    while (traversalBranch.nextLevel) {
                        traversalBranch = traversalBranch.nextLevel;
                    }
                    traversal.endPoint = traversalBranch.vertexTypes ? traversalBranch.vertexTypes[0] : null;
                }
            });
            return traversals;
        };
        this.getTraversalByUuid = (uuid) => {
            return this.adminTraversalsClientService
                .getTraversalByUuid(uuid)
                .pipe(map(result => result.body))
                .toPromise()
                .catch(err => Promise.reject(err?.error?.message));
        };
    }
    getApi() {
        return {
            getFieldsByTraversals: this.getFieldsByTraversals,
            getTraversals: this.getTraversals,
            getTraversalByUuid: this.getTraversalByUuid
        };
    }
};
TraversalApiService = __decorate([
    Injectable()
], TraversalApiService);
export { TraversalApiService };
